from zope.interface import implements, alsoProvides
from zope.component import getUtility
from Products import Five
from Products.CMFCore.utils import getToolByName
from Products.validation import validation

from zope.formlib import form
from Products.Five.formlib import formbase
from zope.schema.fieldproperty import FieldProperty
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.metanav.interfaces import INavigationView, INavigationEdit, \
            INavigationEditFields, INavigationFolderVocabulary, INavigationRemoveView, \
            INavigationPortletView

from zope.app.form.interfaces import WidgetInputError

from zope.component import getMultiAdapter

from zope.schema.vocabulary import SimpleVocabulary
from zope.app.schema.vocabulary import IVocabularyFactory

# support Vocab utility
from zope.component import getUtility
from Products.metanav.interfaces import INavigationFolderVocabulary, SECONDARY_DROPDOWN_MESSAGE
from zope.schema.vocabulary import SimpleVocabulary
from zope.publisher.browser import TestRequest

from zExceptions import BadRequest
from AccessControl import Unauthorized

from Products.metanav.browser.widgets import MetaNavTopLevelFolderWidget, MetaNavSecondaryFolderDrowndownWidget


class NavigationView(Five.BrowserView):
    """See INavigationView
    """
    implements(INavigationView)
    
    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)
        self.catalog = getToolByName(context, 'portal_catalog')

    def getNavigation(self):
        """ Returns a list of dictionaries containing navLevel-1 (from first
        level NavigationFolders).  These each contain a list navLevel-2 (from second
        level NavigationFolders).  These contain a list of navLevel-3 (from actual
        NavigationItems within that folders).
        
        i.e.
        
        [
            {   'id'           : 'nav-level1-1',
                'title'        : 'Nav Level 1',
                'navLevel2'    : [{  'id'       : 'nav-level-2',
                                    'title'    : 'Nav Level 2',
                                    'portal_type' : 'NavigationFolder',
                                    'navLevel3': [{  'id': 'navitem1',
                                                    'title': 'Nav Item 1',
                                                    'url'   : 'http://hello.com'
                                                  }, { next navitem goes here } ]
                                    
                                  }, { next navLevel2 item goes here } ]
            }
            {   'id'           : 'nav-level1-2',
            
                ... etc ...

            }
        ]
        """
        
        root_brains = self.catalog.searchResults(
            portal_type = 'NavigationRootFolder',
        )

        if len(root_brains) == 0:
            raise ValueError, 'We could not find a NavigationRootFolder. Please add one.'
        
        if len(root_brains) > 1:
            raise ValueError, 'We found more than one NavigationRootFolders. We only support \
                one NavigationRootFolder.'

        root = root_brains[0]

        # TODO Add 'NavigationItem' to portal_type in the searches
        #       once code has been refactored to expect NavigationItems
        #       at all levels in the hierarchy

        # grab all level 1 items and stick them in a dictionary
        navLevelOne_brains = self.catalog.searchResults(
                          path = { 'query' : root.getPath(),
                                   'level' :0,
                                   'depth' :1},
                          portal_type = ['NavigationFolder',],
                          sort_on = 'getObjPositionInParent'
                          )
        
        # build navigation hierarchy
        nav = []

        # find any level 1 NavigationFolders
        level1 = []
        for nav_level_one in navLevelOne_brains:
            one = { 'id'       :nav_level_one.id,
                    'uid'      :nav_level_one.UID,
                    'title'    :nav_level_one.Title,
                    'portal_type' :nav_level_one.portal_type,
                    'summary'  :nav_level_one.getSummary,
                    'placement':nav_level_one.getPlacement,
                    'navLevel2':[] }

            # find any level 2 NavigationFolders or NavigationItems
            navLevelTwo_brains = self.catalog.searchResults(
                              path = { 'query' : nav_level_one.getPath(),
                                       'level' :0,
                                       'depth' :1},
                              portal_type = ['NavigationFolder','NavigationItem'],
                              sort_on = 'getObjPositionInParent'
                              )
            for nav_level_two in navLevelTwo_brains:
                if nav_level_two.portal_type == 'NavigationFolder':
                    # add a new NavigationFolder
                    two = { 'id'          :nav_level_two.id,
                            'uid'         :nav_level_two.UID,
                            'title'       :nav_level_two.Title,
                            'portal_type' :nav_level_two.portal_type,
                            'summary'     :nav_level_two.getSummary,
                            'placement'   :nav_level_two.getPlacement,
                            'navLevel3':[] }
                else: 
                    # add a new NavigationItem
                    two = { 'id'          :nav_level_two.id,
                            'title'       :nav_level_two.Title,
                            'portal_type' :nav_level_two.portal_type,
                            'url'         :nav_level_two.navURL,
                            'summary'     :nav_level_two.getSummary,
                            'analyticspath':nav_level_two.getPath(),
                            }
                    
                # find any level 3 (NavigationItems) (only if we're looking at a NavigationFolder)
                if nav_level_two.portal_type == 'NavigationFolder':
                    navLevelThree_brains = self.catalog.searchResults(
                                      path = { 'query' : nav_level_two.getPath(),
                                               'level' :0,
                                               'depth' :1},
                                      portal_type = ['NavigationItem',],
                                      sort_on = 'getObjPositionInParent',
                                      )
                    for nav_level_three in navLevelThree_brains:
                        # only return 'published' and 'pendingremoval' items
                        if nav_level_three.review_state in ('published', \
                            'pendingremoval'):
                            two['navLevel3'].append({   'id'   :nav_level_three.id,
                                                    'title':nav_level_three.Title,
                                                    'portal_type' :nav_level_three.portal_type,
                                                    'url'  : nav_level_three.navURL,
                                                    'summary':nav_level_three.getSummary,
                                                    'analyticspath':nav_level_three.getPath(),
                                                })

                if two:
                  one['navLevel2'].append(two)
            nav.append( one )
            
        return nav

    def getNavigationTopOnly(self):
        """ Returns a filtered version of getNavigation
            (just items marked "top")
        """
        nav = self.getNavigation()
        return [n for n in nav if n.has_key('placement') and n['placement']=='top']

    def getNavigationSideOnly(self):
        """ Returns a filtered version of getNavigation
            (just items marked "side")
        """
        nav = self.getNavigation()
        return [n for n in nav if n.has_key('placement') and n['placement']=='side']
        
class NavigationEdit(formbase.Form):
    """See INavigationEdit
    """
    implements(INavigationEdit)

    #form_fields = form.Fields(INavigationEditFields, omit_readonly=True, render_context=True)
    form_fields = form.Fields(INavigationEditFields)
    form_fields['navigation_location_top_level'].custom_widget = MetaNavTopLevelFolderWidget
    form_fields['navigation_location_top_level_secondary_level'].custom_widget = MetaNavSecondaryFolderDrowndownWidget
    
    #form_fields = form.Fields(IOrder, render_context=True)
    #form_fields = form_fields.omit('now')

    template = ViewPageTemplateFile('navigation_edit.pt')

    status = errors = None
    prefix = 'form'
    
    label = u"Global Navigation Add and Remove Form"
    description = u'This page allows you to submit a navigation link, or remove an existing link.'
    
    actions = form.Actions(
        form.Action('Add', success='handle_add_action',
                        failure='handle_add_action_failure',
                        condition='handle_add_action_condition',
                        validator='handle_add_action_validator'),
    )
    
    def __init__( self, context, request ):
        formbase.Form.__init__(self, context, request)
        self.context = context
        self.request = request
        self.workflow_tool = getToolByName(self.context, 'portal_workflow')
        self.catalog = getToolByName(context, 'portal_catalog')
        #self.setupLocale( request )
        #self.setupEnvironment( request )

    def anyNavigationItemsForContext(self):
        brefs = self.context.getBRefs(relationship='GlobalMenuTo')
        if brefs:
            return True
        else:
            return False        
    
    def getSubNavigationFoldersXML(self, topLevelFolderTitle):
        """ Returns XML use by javascript to populate secondary navigation
        dropdown widget.
        
        FIXME: The reason we're passing a folder title (topLevelFolderTitle) instead
        of a folder UID is that formlib's Choice() field uses Title for the key
        then does some magic and translates that to UID for formlib to process
        when the form is submitted.  Need to resolve that issue first.
        """

        root_brains = self.catalog.searchResults(portal_type = 'NavigationRootFolder')

        if len(root_brains) == 0:
            raise ValueError, 'We could not find a NavigationRootFolder. Please add one.'
        
        if len(root_brains) > 1:
            raise ValueError, 'We found more than one NavigationRootFolders. We only support \
                one NavigationRootFolder.'

        root = root_brains[0]

        topLevelFolders = navLevelOne_brains = self.catalog.searchResults(
                          path = { 'query' : root.getPath(),
                                   'level' :0,
                                   'depth' :1},
                          portal_type = ['NavigationFolder',],
                          Title = topLevelFolderTitle
                          )
        subFolders = []
        if len(topLevelFolders) > 0:
            # TODO: There really should only be one, but since it's possible
            #   more than one folder will have same title, we're just
            #   grabbing the first one.
            top_folder_path = topLevelFolders[0].getPath()
            subFolders = self.catalog.searchResults(
                              path = { 'query' : top_folder_path,
                                       'level' :0,
                                       'depth' :1},
                              portal_type = ['NavigationFolder',],
                              sort_on = 'getObjPositionInParent'
                              )
        
        RESPONSE = self.request.RESPONSE
        RESPONSE.setHeader('Content-Type', 'text/xml')

        def beefystrip(s):

          charEscapes = {
            '<':'&lt;',
            '>':'&gt;',
            '"':'&#34;',
            '\'':'&#39;',
            '-':'&#45;',
            '_':'&#95;',
            ':':'&#58;',
            '/':'&#47;',
            '(':'&#40;',
            ')':'&#41;',
            '.':'&#46;',
            ',':'&#44;',
            '!':'&#33;',
            '?':'&#63;',
            '%':'&#37;',
            '$':'&#36;',
            '=':'&#61;',
            '+':'&#43;',
            '*':'&#42;',
            '@':'&#64;',
            'ZAMPERSAND':'&#38;',
            'ZPOUND':'&#35;',
            'ZSEMICOLON':'&#59;',
            'ZAPOSTROPHE':'&#39;',
          }

          a = "ZAMPERSAND"
          b = "ZPOUND"
          c = "ZSEMICOLON"
          d = "ZAPOSTROPHE"
          s = s.replace('&', a)
          s = s.replace('#', b)
          s = s.replace(';', c)
          s = s.replace("'", d)

          for key in charEscapes:
            s = s.replace(key, charEscapes[key])

          if s:
            new_s = list(s)
            punctuation = ['&', ';', '#', "'"]
            return "".join([c for c in new_s if c.isalnum() or c.isspace() or c in punctuation])
          return s


        xml = """<?xml version="1.0" encoding="UTF-8"?><results>"""
        xml += '<result id="none" title="&lt;no selection&gt;"></result>'

        if subFolders:
            for item in subFolders:
                id = beefystrip(item.UID)
                title = beefystrip(item.Title)
                xml += '<result id="%s" title="%s"></result>' % (id, title)

        xml += "</results>"

        return xml

    def getNavigationItems(self):
        brefs = self.context.getBRefs(relationship='GlobalMenuTo')
        if brefs:
            return [{'title':bref.Title(), \
                     'uid'  :bref.UID(), \
                     'review_state': self.workflow_tool.getInfoFor(bref, 'review_state') \
                     } for bref in brefs]
        
    def setUpWidgets(self, ignore_request=False):
        # data to initialize the form with
        data = {}
        
        self.widgets = form.setUpWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            data=data,
            ignore_request=ignore_request
            )

    #@form.action("Request MetaNav", failure='handle_edit_action_failure')
    def handle_add_action(self, action, data):
        # grab folder by UID
        folder_top = data['navigation_location_top_level']
        folder_secondary = data['navigation_location_top_level_secondary_level']
        if folder_secondary is None or folder_secondary in (u'none', SECONDARY_DROPDOWN_MESSAGE):
            uid = folder_top
        else:
            uid = folder_secondary
        catalog = getToolByName(self.context, 'uid_catalog')        
        folder = catalog.searchResults(UID = uid)[0].getObject()

        # add a new NavigationItem
        plone_tool = getToolByName(self.context, 'plone_utils', None)
        clean_id = plone_tool.normalizeString(data['navigation_item_title'])
        clean_title = data['navigation_item_title'].encode('ascii')
        if data.has_key('navigation_item_external_url') and data['navigation_item_external_url'] is not None:
            clean_externalurl = data['navigation_item_external_url'].encode('ascii')
        else:
            clean_externalurl = ''
        
        # Check to see if ID already exists.  If it does, add a number to it
        increment = 1
        non_dup_id = clean_id
        while non_dup_id in [f.id for f in folder.getFolderContents()]:
            non_dup_id = "%s-%s" % (clean_id, increment)
            increment += 1
        clean_id = non_dup_id

        try:
            folder.invokeFactory('NavigationItem', id=clean_id, \
                    title=clean_title, summary=data['navigation_item_summary'],
                    externalurl=clean_externalurl)
        except BadRequest,e:
            self.status = "Update FAILED due to BadRequest '%s'" % e
            return
        except Unauthorized, e:
            # what if user doesn't have permissions?
            self.status = "Update FAILED due to Unauthorized '%s'" % e
            return
        except:
            self.status = 'Update FAILED for an unexpected reason...'
            return
        
        # Add a reference if there isn't an externalurl
        if not clean_externalurl:
            folder.restrictedTraverse(clean_id).addReference(self.context,"GlobalMenuTo")

        self.status = 'Updated successfully.'

        # removed form.applyChanges() call since we don't
        # want to save to the adapter (for the content-type)
        
        #if form.applyChanges(self.context, self.form_fields, data):
        #    self.status = 'Object updated'
        #else:
        #    self.status = 'No changes'
        

    def handle_add_action_failure(self, action, data, errors):
        self.status = 'There were %d errors.' % len(errors)
        
    def handle_add_action_validator(self, action, data):
        errors = []
        errors.extend( 
                form.getWidgetsData(self.widgets, self.prefix, data) +
                form.checkInvariants(self.form_fields, data)
        )

        # check that top-level nav is not 'none'
        navtoplevel='none'
        try:
            navtoplevel = self.widgets['navigation_location_top_level'].getInputValue()
        except:
            pass
        if 'none' == navtoplevel:
            err_msg = u"Please choose a button. NOTE: You may need to re-select your category."
            self.widgets['navigation_location_top_level'].error = err_msg
            # set Widget error notice at top of form
            errors.append(WidgetInputError(
                field_name='navigation_location_top_level',
                widget_title=u'Step 1',
                errors=err_msg),
            )
            
        # check external URL if one was added
        if data.has_key('navigation_item_external_url'):
            if data['navigation_item_external_url'] is not None:
                v = validation.validatorFor('isURL')
                if v(data['navigation_item_external_url'].encode('ascii')) <> 1:
                    # set widget error
                    err_msg = u"'%s' is an invalid URL" % data['navigation_item_external_url']
                    self.widgets['navigation_item_external_url'].error = err_msg
                    # set Widget error notice at top of form
                    errors.append(WidgetInputError(
                        field_name='navigation_item_external_url',
                        widget_title=u'External site URL',
                        errors=err_msg),
                    )
        return errors

    def handle_add_action_condition(self, action):
        # always show button since we can add one or more nav links
        return True

class NavigationRemoveView(object):
    """ See INavigationRemoveView """
    
    implements(INavigationRemoveView)
    
    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)

    def __call__(self, *args, **kwargs):
        self.workflow_tool = getToolByName(self.context, 'portal_workflow')
        if self.request.form.has_key('uid'):
            nav_uid = self.request.form['uid']
        
            # is NavigationItem's UID valid?
            uid_catalog = getToolByName(self.context, 'uid_catalog')
            
            # TODO Extra check: Is there a relationship between context and our NavigationItem?
            pass
        
            result = uid_catalog.searchResults(UID = nav_uid)
            if result and len(result)==1:
                navitem = result[0].getObject()
        
                # set workflow to 'requestremoval'
                review_state = self.workflow_tool.getInfoFor(navitem, 'review_state')
                if review_state == 'published':
                    self.workflow_tool.doActionFor(navitem, 'requestremoval', \
                        comment='Content editor requested removal of this navigation item')
                    
        self.request.response.redirect("%s/%s" % (self.context.absolute_url(), "@@navigation_edit_form"))
        
class NavigationPortletView(object):
    """ See INavigationPortletView """
    
    implements(INavigationPortletView)
    
    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)
        self.catalog = getToolByName(context, 'portal_catalog')
        
    def getNavItemsForReview(self):
        navitems = self.catalog.searchResults(
                          portal_type = ['NavigationItem',],
                          )
        return [n for n in navitems if n.review_state in ('pending', 'pendingremoval')]

class NavigationFolderVocabularyFromNavRoot(object):
    """ A utility that returns all NavigationFolders underneath
        NavigationRootFolder singleton, using its UID as the unique
        identifier.

        e.g.
        
        [('Nav Level 1 Folder A', '26a7cfd3338e7643f5ff0b4c1fd9553e'),
         ('Nav Level 1 Folder A / Nav Level 2 Folder A ', 'dfc83e535aeb4d0b9d36d7e76d018c94'),
         ('Nav Level 1 Folder B', '24f9358387abd984fef7c26cce90de4f'),
         ('Nav Level 1 Folder B / Nav Level 2 Folder B ', '32f4a9f56a753820a166e4d7d34d5079')]

    """
    implements(INavigationFolderVocabulary)

    #@property
    def navigation_folders(self, context, request):
        
        view = getMultiAdapter((context, request), name='navigation_browserview')
        nav = view.getNavigation()
        
        # convert nav into a sorted list with reference_field as the ID
        folders = [(u'Please choose a button...', u'none')]
        for one in nav:
            folders.append((one['title'], one['uid'], ),)
        return folders
    
def navigationFolderVocabularyTopLevel(context):
    """ Vocabulary Factory """
    utility = getUtility(INavigationFolderVocabulary)
    return SimpleVocabulary.fromItems(utility.navigation_folders(context,context.request))

alsoProvides(navigationFolderVocabularyTopLevel, IVocabularyFactory)
