
from zope.app.form.browser.itemswidgets import ItemsEditWidgetBase
from zope.app.i18n import ZopeMessageFactory as _
from zope.app.form.browser.widget import SimpleInputWidget, renderElement
from zope.schema.vocabulary import SimpleVocabulary
from zope.schema import Choice

class ChoiceNoValidation(Choice):
    def __init__(self, values=None, vocabulary=None, source=None, **kw):
        Choice.__init__(self, values=values, vocabulary=vocabulary, source=source, **kw)
        
    def _validate(self, value):
        return True

class MetaNavTopLevelFolderWidget(ItemsEditWidgetBase):
    """ This Widget is similar to the DropdownWidget, except that it also
        includes onclick='getSubFolders(this)' javascript to populate
        form.navigation_location_top_level_secondary_level field.
    
        FIXME: Also fixes issue with select "value" being same as "token"
        so if two tokens are the same, the default DropdownWidget
        blows up.  (We have our own renderItemsWithValues())
    """

    size = 1
    
    _messageNoValue = _("vocabulary-missing-single-value-for-edit",
                        "(no value)")

    def __init__(self, field, request):
        ItemsEditWidgetBase.__init__(self, field, field.vocabulary, request)

    def renderValue(self, value):
        rendered_items = self.renderItems(value)
        contents = "\n%s\n" %"\n".join(rendered_items)
        return renderElement('select',
                             name=self.name,
                             id=self.name,
                             contents=contents,
                             size=self.size,
                             extra=self.extra,
                             onclick='getSubFolders(this)')

    def renderItemsWithValues(self, values):
        """Render the list of possible values, with those found in
        `values` being marked as selected."""

        cssClass = self.cssClass

        # multiple items with the same value are not allowed from a
        # vocabulary, so that need not be considered here
        rendered_items = []
        count = 0

        # Handle case of missing value
        missing = self._toFormValue(self.context.missing_value)

        if self._displayItemForMissingValue and not self.context.required:
            if missing in values:
                render = self.renderSelectedItem
            else:
                render = self.renderItem

            missing_item = render(count,
                self.translate(self._messageNoValue),
                missing,
                self.name,
                cssClass)
            rendered_items.append(missing_item)
            count += 1

        # Render normal values
        for term in self.vocabulary:
            item_text = self.textForValue(term)

            if term.value in values:
                render = self.renderSelectedItem
            else:
                render = self.renderItem

            rendered_item = render(count,
                item_text,
                term.token,
                self.name,
                cssClass)

            rendered_items.append(rendered_item)
            count += 1

        return rendered_items

    def renderItems(self, value):
        return self.renderItemsWithValues([value])

class MetaNavSecondaryFolderDrowndownWidget(ItemsEditWidgetBase):
    """ This Widget is similar to the DropdownWidget, except that it
    overrides _toFieldValue to not validate against initial vocabulary
    since we dynamically fill the Dropdown via Javascript.
    """

    size = 1
    
    _messageNoValue = _("vocabulary-missing-single-value-for-edit",
                        "(no value)")

    def __init__(self, field, request):
        ItemsEditWidgetBase.__init__(self, field, field.vocabulary, request)

    def _toFieldValue(self, input):
        if input:
            return input
        else:
            return self.context.missing_value
            
    def renderValue(self, value):
        rendered_items = self.renderItems(value)
        contents = "\n%s\n" %"\n".join(rendered_items)
        return renderElement('select',
                             name=self.name,
                             id=self.name,
                             contents=contents,
                             size=self.size,
                             extra=self.extra,)

    def renderItemsWithValues(self, values):
        """Render the list of possible values, with those found in
        `values` being marked as selected."""

        cssClass = self.cssClass

        # multiple items with the same value are not allowed from a
        # vocabulary, so that need not be considered here
        rendered_items = []
        count = 0

        # Handle case of missing value
        missing = self._toFormValue(self.context.missing_value)

        if self._displayItemForMissingValue and not self.context.required:
            if missing in values:
                render = self.renderSelectedItem
            else:
                render = self.renderItem

            missing_item = render(count,
                self.translate(self._messageNoValue),
                missing,
                self.name,
                cssClass)
            rendered_items.append(missing_item)
            count += 1

        # Render normal values
        for term in self.vocabulary:
            item_text = self.textForValue(term)

            if term.value in values:
                render = self.renderSelectedItem
            else:
                render = self.renderItem

            rendered_item = render(count,
                item_text,
                term.token,
                self.name,
                cssClass)

            rendered_items.append(rendered_item)
            count += 1

        return rendered_items

    def renderItems(self, value):
        return self.renderItemsWithValues([value])