"""metanav -- NavigationItem"""

__author__  = 'Brian Gershon <briang@webcollective.coop>'
__docformat__ = 'plaintext'

try:
    from Products.LinguaPlone.public import *
except ImportError:
    from Products.Archetypes.public import *

## General
from DateTime import DateTime

## Zope
from zope.interface import implements

## CMF
from Products.CMFCore.permissions import ModifyPortalContent

## ATCT + Friends
from Products.ATContentTypes.content.document import ATDocumentSchema, ATDocument
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

##Archetypes
#from Products.Archetypes import atapi

## Product
from Products.metanav.interfaces import INavigationItem
from Products.metanav.config import PROJECTNAME

#V_REQUIRED   = 1
#V_SUFFICIENT = 2

NavigationItemSchema = ATDocumentSchema.copy() + Schema((

    StringField(
        name='summary',
        widget=StringWidget
        (
            label="Summary",
            description="Brief 5-10 word summary",
        ),
    ),

    ReferenceField(
        name='contentpointer',
        widget=ReferenceWidget(
            description="Content this menu item points to",
            label='Points to',
        ),
        relationship="GlobalMenuTo",
        vocabulary_display_path_bound=-1,    #remove long path
    ),

    StringField(
        name='externalurl',
        validators=('isURL'),
        # for testing: validators=(('isEmpty', V_SUFFICIENT), ('isURL', V_REQUIRED)),
        widget=StringWidget(
            label="External URL",
            description="A URL in this field overrides the URL generated from the 'Points to' field above.",
        ),
    ),

))

#NavigationItemSchema['text'].required = False

finalizeATCTSchema(NavigationItemSchema)

class NavigationItem(ATDocument):
    """A Navigation Item"""
    implements(INavigationItem)
    __implements__ = (getattr(ATDocument,'__implements__',()),)
    
    schema         =  NavigationItemSchema
    
    _at_rename_after_creation = True

    suppl_views = ()
    
    typeDescription= """A Navigation Item"""
    
    def navURL(self):
        """ Populate new navURL catalog field.

        If externalurl, return that.  Otherwise, get the content
        referenced by our contentpointer field, and return its URL.
        """
        if self.getExternalurl():
            return self.getExternalurl()
        else:
            refs = self.getRefs(relationship='GlobalMenuTo')
            if len(refs) == 1:
                url = refs[0].absolute_url()
                return url
            else:
                return None
        

registerATCT(NavigationItem, PROJECTNAME)
