from zope.interface import Interface
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary
from Products.metanav.browser import widgets

class INavigationRootFolder(Interface):
    """NavigationRootFolder contractual interface
    """

class INavigationFolder(Interface):
    """NavigationFolder contractual interface
    """

class INavigationItem(Interface):
    """NavigationItem contractual interface
    """

class INavigationView(Interface):
    """NavigationView browser view
    """
    
    def getNavigation():
        """ Returns a list of dictionaries containing navLevel-1 (from first
        level NavigationFolders).  These each contain a list navLevel-2 (from second
        level NavigationFolders).  These contain a list of navLevel-3 (from actual
        NavigationItems within that folders)."""
        
    def getNavigationTopOnly():
        """ Returns a filtered version of getNavigation
            (just items marked "top")
        """

    def getNavigationSideOnly():
        """ Returns a filtered version of getNavigation
            (just items marked "side")
        """


class INavigationRemoveView(Interface):
    """ View for setting NavigationItems to 'pendingremoval'
        workflow state based on UID of NavigationItem.
    """

class INavigationPortletView(Interface):
    """ See INavigationPortletView """

    def getNavItemsForReview():
        """ Return list of NavigationItem brains that are in the
        'pending' or 'pendingremoval' state. """
        
class INavigationEdit(Interface):
    """NavigationEdit browser view
    """
    
    def anyNavigationItemsForContext():
        """ Returns True if we have any NavigationItems connected with
        current context.  To retrieve a dictionary of information,
        use the getNavigationItems() method.
        
        Used by Add Action.
        """
        
    def getSubNavigationFoldersXML(topLevelFolderTitle):
        """ comments in browser.py """
        
    def getNavigationItems():
        """ Returns a list of dictionaries containing the information
        from NavigationItem(s) if they exist for this context
        otherwise, return an empty list.
        """
        
    def setUpWidgets():
        """ Create widgets and fill with default values
        
            navigation_location_top_level is handled directly via vocabulary utility.
        """
        
    def handle_add_action(action, data):
        """ Create a new NavigationItem in the proper place based on
        navigation_location_top_level.  Giving it a title and summary.
        """

    def handle_add_action_failure(action, data, errors):
        """"""
        
    def handle_add_action_validator(action, data):
        """ Validate fields during an Add action
        
            field data comes in from self.request
            
            (I think data is just for extra data,
                not the field data)
        """

    def handle_add_action_condition(action):
        """ If there are no current metanav items for this piece of
        content, show the add Action button.
        """

SECONDARY_DROPDOWN_MESSAGE='Please choose a category...'

class INavigationEditFields(Interface):
    """ Fields in the navigation edit form """

    navigation_location_top_level = schema.Choice(
                title = u"Step 1: Choose a button under which your page will be found",
                description = u"""""",
                required = True,
                vocabulary = 'NavigationFolder Vocabulary Top Level',   # Vocabulary Factory
                )

    navigation_location_top_level_secondary_level = widgets.ChoiceNoValidation(
                title = u"Step 2: Choose a category",
                description = u"""If you desire, choose a category under that button in which to link your page. If you want your page to be linked from the list of categories, leave this field blank.""",
                required = True,    # needs to be required so that drop-down doesn't auto-create a non-value entry
                # vocabulary dynamically filled at runtime
                vocabulary= SimpleVocabulary.fromItems([(SECONDARY_DROPDOWN_MESSAGE,'none')])
                )

    navigation_item_title = schema.TextLine(
                title = u"Step 3: Navigation title",
                description=u"This is the text that shows up in the navigation.",
                required = True,
                #default = u""
                )

    navigation_item_summary = schema.TextLine(
                title = u"Navigation brief summary",
                description=u"This is the text that shows up when hoving over the " +
                            u"item in the navigation.",
                required = False,
                #default = u""
                )

    navigation_item_external_url = schema.TextLine(
                title = u"Link to content on an external site",
                description=u"If this field is filled in with a URL (e.g. http://somelink.com), the new global navigation link will point to the URL and not to this piece of content you are currently viewing. Note: Please double-check your link since you won't be able to remove it yourself and would need to have a site manager remove it.",
                required = False,
                )

class INavigationFolderVocabulary(Interface):
    """ A utility that returns a vocabulary of NavigationFolders """
    navigation_folders = schema.Iterable(
        title=u'NavigationFolder Vocabulary Top Level',
        description=u'A utility that returns a vocabulary of NavigationFolders')