# Integration tests specific to content-types
#

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.metanav.tests import base
from Products.CMFCore.utils import getToolByName

# zope
from zope.interface.verify import verifyClass
from DateTime import DateTime
from zope.publisher.browser import TestRequest

# interfaces
from Products.metanav.interfaces import INavigationFolder, INavigationItem
from Products.ATContentTypes.interfaces import IATFolder, IATNewsItem

# content objects
from Products.metanav.content import navigationfolder, navigationitem

class TestContentAddingEditing(base.MetaNavTestCase):
    """ Ensure that our custom content objects are editable and correctly 
        registered via the component architecture with interfaces.
    """
    def afterSetUp(self):
        # add some content objects
        self.setupAllTypes()
        
        self.workflow_tool = self.portal.portal_workflow
        self.portal.acl_users._doAddUser('navigationmanager', 'secret', ['Member',  \
            'NavigationManager'], [])

        self.membership = self.portal.portal_membership
        self.member = self.membership.getMemberById('test_user_1_')
        #print self.membership.getAuthenticatedMember()

    def testSetGetNavigationRootFolder(self):
        """Our afterSetUp step confirms that we can add the actual 
           content-type.  Now we confirm its API for set/get.
        """
        # make sure it's there
        self.failUnless('navigationrootfolder' in self.folder.objectIds())
        
        # set a bunch of fields
        self.folder.navigationrootfolder.setTitle("""Title""")
        self.folder.navigationrootfolder.setDescription("""Description""")
        
        # get a bunch of the values
        self.assertEqual(self.folder.navigationrootfolder.Title(),"""Title""")
        self.assertEqual(self.folder.navigationrootfolder.Description(),"""Description""")
    
    def testSetGetNavigationFolder(self):
        """Our afterSetUp step confirms that we can add the actual 
           content-type.  Now we confirm its API for set/get.
        """
        # make sure it's there
        self.failUnless('navigationfolder' in self.folder.objectIds())
        
        # set a bunch of fields
        self.folder.navigationfolder.setTitle("""Title""")
        self.folder.navigationfolder.setDescription("""Description""")
        self.folder.navigationfolder.setSummary("""Brief summary goes here""")
        
        # get a bunch of the values
        self.assertEqual(self.folder.navigationfolder.Title(),"""Title""")
        self.assertEqual(self.folder.navigationfolder.Description(),"""Description""")
        self.assertEqual(self.folder.navigationfolder.getSummary(),"""Brief summary goes here""")

    def testSetGetNavigationItem(self):
        """Our afterSetUp step confirms that we can add the actual 
           content-type.  Now we confirm its API for set/get.
        """
        # make sure it's there
        self.failUnless('navigationitem' in self.folder.objectIds())
        
        # set a bunch of fields
        self.folder.navigationitem.setTitle("""Title""")
        self.folder.navigationitem.setDescription("""Description""")
        self.folder.navigationitem.setText("""<p>BodyText</p>""")
        
        # get a bunch of the values
        self.assertEqual(self.folder.navigationitem.Title(),"""Title""")
        self.assertEqual(self.folder.navigationitem.Description(),"""Description""")
        self.assertEqual(self.folder.navigationitem.getText(),"""<p>BodyText</p>""")

        # Test 'externalurl'
        
        # Content starts out in pending workflow. Owners should have perms to update, as well
        #   as NavigationManagers.
        self.assertEqual(self.workflow_tool.getInfoFor(self.folder.navigationitem, 'review_state'), 'pending',
            "NavigationItem should start as 'pending'")        
        self.folder.navigationitem.update(externalurl = 'http://www.hello999.com')
        self.assertEqual(self.folder.navigationitem.getExternalurl(),"""http://www.hello999.com""")

        ## try an invalid URL field
        ## TODO: isURL validator validates to blank, then returns true without checking isURL
        ##   due to isURL adding an ('isEmpty', V_SUFFICIENT) so test not working.
        #request = TestRequest(form={ 'fieldset':'default', 'externalurl':'crap',})
        #errors = {}
        #schema = self.folder.navigationitem.Schema()
        #schema.validate(instance=self.folder.navigationitem, REQUEST=request,
        #             errors=errors, data=None, metadata=None)
        #print errors
        #field = self.folder.navigationitem.Schema()['externalurl']
        #field.validate('', self.folder.navigationitem)
        #self.failUnless(field.validate('hello555com', self.folder.navigationitem) is not None,'This should have failed')
        #self.failUnless(field.validate('', self.folder.navigationitem) is None,'Blank URL should not have failed')
        #self.failUnless(field.validate('http://www.webcollective.coop/', self.folder.navigationitem) is None,'Valid URL should not have failed')
        
        self.login('navigationmanager')
        self.folder.navigationitem.setExternalurl("""http://www.hello555.com""")
        self.assertEqual(self.folder.navigationitem.getExternalurl(),"""http://www.hello555.com""")
        #print self.member.getRolesInContext(self.folder.navigationitem)
        #self.assertEqual(sortTuple(member.getRolesInContext(self.folder)),
        #                 ('Authenticated', 'Foo', 'Member'))
        
    
if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestContentAddingEditing))
    return suite
