# Integration tests specific to our eCards product
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.metanav.tests import base
from Products.CMFCore.utils import getToolByName

from Products.metanav.interfaces import INavigationFolder, INavigationItem, \
    INavigationRootFolder, INavigationView, INavigationEdit, INavigationRemoveView, \
    INavigationPortletView
from zope.interface.verify import verifyObject, verifyClass

from Products.metanav.config import ALLTYPES
from Products.metanav.content.navigationrootfolder import NavigationRootFolder
from Products.metanav.content.navigationfolder import NavigationFolder
from Products.metanav.content.navigationitem import NavigationItem
from Products.metanav.browser.browser import NavigationView
from Products.metanav.browser.browser import NavigationEdit
from Products.metanav.browser.browser import NavigationRemoveView
from Products.metanav.browser.browser import NavigationPortletView

class TestProductInterfaces(base.MetaNavTestCase):
    """ Ensure that our metanav product classes and objects
        fullfill their contractual interfaces
    """
    def afterSetUp(self):
        pass
    
    def testContentInterfaces(self):
        """ Some basic boiler plate testing of Interfaces and objects"""
        self.failUnless(INavigationItem.implementedBy(NavigationItem))
        self.failUnless(verifyClass(INavigationItem, NavigationItem))
    
        self.failUnless(INavigationFolder.implementedBy(NavigationFolder))
        self.failUnless(verifyClass(INavigationFolder, NavigationFolder))

        self.failUnless(INavigationRootFolder.implementedBy(NavigationRootFolder))
        self.failUnless(verifyClass(INavigationRootFolder, NavigationRootFolder))
    
    def testObjectInstances(self):
        # setup our content objects
        self.setupAllTypes()
        
        # verify we're an object of the expected class
        self.failUnless(isinstance(self.folder.navigationrootfolder, NavigationRootFolder))
        self.failUnless(isinstance(self.folder.navigationfolder, NavigationFolder))
        self.failUnless(isinstance(self.folder.navigationitem, NavigationItem))
        
    def testContentObjectsVerify(self):
        # setup our content objects
        self.setupAllTypes()

        self.failUnless(verifyObject(INavigationRootFolder, self.folder.navigationrootfolder))
        self.failUnless(verifyObject(INavigationFolder, self.folder.navigationfolder))
        self.failUnless(verifyObject(INavigationItem, self.folder.navigationitem))
    
    def testBrowserViewInterfaces(self):
        self.failUnless(INavigationView.implementedBy(NavigationView))
        self.failUnless(verifyClass(INavigationView, NavigationView))

        self.failUnless(INavigationEdit.implementedBy(NavigationEdit))
        self.failUnless(verifyClass(INavigationEdit, NavigationEdit))

        self.failUnless(INavigationRemoveView.implementedBy(NavigationRemoveView))
        self.failUnless(verifyClass(INavigationRemoveView, NavigationRemoveView))

        self.failUnless(INavigationPortletView.implementedBy(NavigationPortletView))
        self.failUnless(verifyClass(INavigationPortletView, NavigationPortletView))
    
    def testBrowserViewsVerify(self):
        # get our view and verify we're an object of the expected class
        view = self.folder.restrictedTraverse('navigation_browserview')
        self.failUnless(isinstance(view, NavigationView))
        self.failUnless(verifyObject(INavigationView, view))

        view = self.folder.restrictedTraverse('navigation_edit_form')
        self.failUnless(isinstance(view, NavigationEdit))
        self.failUnless(verifyObject(INavigationEdit, view))

        view = self.folder.restrictedTraverse('navigation_remove')
        self.failUnless(isinstance(view, NavigationRemoveView))
        self.failUnless(verifyObject(INavigationRemoveView, view))

        view = self.folder.restrictedTraverse('navigation_portlet')
        self.failUnless(isinstance(view, NavigationPortletView))
        self.failUnless(verifyObject(INavigationPortletView, view))

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductInterfaces))
    return suite
