#
# Test roles and workflow
#

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.metanav.tests import base
from Products.CMFCore.utils import getToolByName

# zope
from zope.interface.verify import verifyClass
from DateTime import DateTime
from AccessControl import Unauthorized

# interfaces
from Products.metanav.interfaces import INavigationFolder, INavigationItem, INavigationPortletView
from Products.ATContentTypes.interfaces import IATFolder, IATNewsItem

# content objects
from Products.metanav.content import navigationfolder, navigationitem

from zope.publisher.browser import TestRequest
from zope.component import getMultiAdapter

from zope.component import getUtility
from Products.metanav.interfaces import INavigationFolderVocabulary
from zope.schema.vocabulary import SimpleVocabulary

class TestRolesAndWorkflow(base.MetaNavTestCase):
    """ Ensure that our roles and workflows are working correctly.
    """

    def afterSetUp(self):
        self.workflow_tool = getToolByName(self.portal, 'portal_workflow')
        self.portal_catalog = getToolByName(self.portal, 'portal_catalog')
        self.portal.acl_users._doAddUser('manager', 'secret', ['Member', 'Manager'], [])
        self.portal.acl_users._doAddUser('navigationmanager', 'secret', ['Member',  \
            'NavigationManager'], [])
        self.portal.acl_users._doAddUser('navigationmanager2', 'secret', ['Member', \
            'NavigationManager'], [])
        
        # create a root folder
        self.login('manager')
        self.portal.portal_types.NavigationRootFolder.global_allow = 1
        self.portal.invokeFactory('NavigationRootFolder', id='navigation_root_folder')
        self.portal.portal_types.NavigationRootFolder.global_allow = 0
        self.rootnavfolder = self.portal.restrictedTraverse('navigation_root_folder')
    
    def testNavigationManagerRole(self):
        """ Did we add our new NavigationManager role successfully?
        """
        roles = list(self.portal.validRoles())
        self.failUnless( 'NavigationManager' in roles, 'Missing NavigationManager role')

    def testWorkflowInstall(self):
        workflowlist = ('navigationfolder_workflow', 'navigationitem_workflow',
                        'folder_workflow', 'plone_workflow',)
        
        for w in workflowlist:
            self.failUnless(w in self.workflow_tool.getWorkflowIds(),
                            '%s workflow not installed' % w)

    # TODO
    # Removed this test since Anonymous can currently see these.
    # Add this back if we want to officially hide these via
    # 'navigationfolder_workflow' tweaks.
    #
    #def testAnonymous_NoAccess(self):
    #    """ Test our NavigationFolder workflow functionality
    #        for Anonymous users.  They should have no access."""
    #    # create a NavigationFolder
    #    self.login('navigationmanager')
    #    self.rootnavfolder.invokeFactory('NavigationFolder', id='navigation_folder')
    #    navfolder = self.rootnavfolder.restrictedTraverse('navigation_folder')
    #    # become anonymous
    #    self.logout()
    #    self.assertRaises(AttributeError, self.portal.restrictedTraverse, 'navigation_folder')
    
    def _createNavigation(self):
        """ Helper function for creating a test Navigation hierarchy that is used
        in many of the tests.
        
        Creates:
        
        rootnavfolder -- nav_level_1_folder_A         -- nav_level_1_folder_B
                         |                               |
                         -- nav_item_5_A                 |
                         -- nav_item_6_A                 |
                         |                               |
                         -- nav_level_2_folder_A         -- nav_level_2_folder_B
                            |                            |
                            -- nav_item_1_A                -- nav_item_1_B
                            -- nav_item_2_A                -- nav_item_2_B
                            -- nav_item_3_A                -- nav_item_3_B
                            -- nav_item_4_A (an external URL)
        """
        
        self.rootnavfolderPython = [
            {   'id':'nav_level_1_folder_A', 'title':'',
                'navLevel2':[
                    {   'id':'nav_level_2_folder_A',
                        'title':'',
                        'navLevel3': [
                            {'id':'nav_item_1_A','title':'', 'url':''},
                            {'id':'nav_item_2_A','title':'', 'url':''},
                            {'id':'nav_item_3_A','title':'', 'url':''},
                            {'id':'nav_item_4_A','title':'', 'url':''}
                        ]
                    }]
            },
            {   'id':'nav_level_1_folder_B', 'title':'',
                'navLevel2':[
                    {   'id':'nav_level_2_folder_B',
                        'title':'',
                        'navLevel3': [
                            {'id':'nav_item_1_B','title':'', 'url':''},
                            {'id':'nav_item_2_B','title':'', 'url':''},
                            {'id':'nav_item_3_B','title':'', 'url':''}
                        ]
                    }]
            },

        ]

        self.login('navigationmanager')
        
        # Create top level "A"
        self.rootnavfolder.invokeFactory('NavigationFolder', id='nav_level_1_folder_A',
                title="Nav Level 1 Folder A")
        self.nav_level_1_folder_A = self.rootnavfolder.restrictedTraverse('nav_level_1_folder_A')

        self.nav_level_1_folder_A.invokeFactory('NavigationFolder', id='nav_level_2_folder_A',
                title="Nav Level 2 Folder A")
        self.nav_level_2_folder_A = self.nav_level_1_folder_A.restrictedTraverse('nav_level_2_folder_A')

        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='nav_item_1_A',
                title="Nav Item 1 A")
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='nav_item_2_A',
                title="Nav Item 2 A")
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='nav_item_3_A',
                title="Nav Item 3 A")
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='nav_item_4_A',
                title="Nav Item 4 A", externalurl="http://www.webcollective.coop")
        self.nav_item_1_A = self.nav_level_2_folder_A.restrictedTraverse('nav_item_1_A')
        self.nav_item_2_A = self.nav_level_2_folder_A.restrictedTraverse('nav_item_2_A')
        self.nav_item_3_A = self.nav_level_2_folder_A.restrictedTraverse('nav_item_3_A')
        
        # TEST THAT FIELDS SET HERE SHOW UP IN THE VIEW LATER ON IN testNavigationView()
        self.nav_item_4_A = self.nav_level_2_folder_A.restrictedTraverse('nav_item_4_A')
        self.nav_item_4_A.setExternalurl("http://www.webcollective.coop")

        # Add NavigationItems underneath level one folders (so they are in level2
        #   along with NavigationFolders)
        self.nav_level_1_folder_A.invokeFactory('NavigationItem', id='nav_item_5_A',
                title="Nav Item 5 A")
        self.nav_level_1_folder_A.invokeFactory('NavigationItem', id='nav_item_6_A',
                title="Nav Item 6 A")
        self.nav_item_5_A = self.nav_level_2_folder_A.restrictedTraverse('nav_item_5_A')
        self.nav_item_6_A = self.nav_level_2_folder_A.restrictedTraverse('nav_item_6_A')
        
        # Create top level "B"
        self.rootnavfolder.invokeFactory('NavigationFolder', id='nav_level_1_folder_B',
                title="Nav Level 1 Folder B")
        self.nav_level_1_folder_B = self.rootnavfolder.restrictedTraverse('nav_level_1_folder_B')

        self.nav_level_1_folder_B.invokeFactory('NavigationFolder', id='nav_level_2_folder_B',
                title="Nav Level 2 Folder B")
        self.nav_level_2_folder_B = self.nav_level_1_folder_B.restrictedTraverse('nav_level_2_folder_B')

        self.nav_level_2_folder_B.invokeFactory('NavigationItem', id='nav_item_1_B',
                title="Nav Item 1 B")
        self.nav_level_2_folder_B.invokeFactory('NavigationItem', id='nav_item_2_B',
                title="Nav Item 2 B")
        self.nav_level_2_folder_B.invokeFactory('NavigationItem', id='nav_item_3_B',
                title="Nav Item 3 B")
        self.nav_item_1_B = self.nav_level_2_folder_B.restrictedTraverse('nav_item_1_B')
        self.nav_item_2_B = self.nav_level_2_folder_B.restrictedTraverse('nav_item_2_B')
        self.nav_item_3_B = self.nav_level_2_folder_B.restrictedTraverse('nav_item_3_B')
        
    def testNavigationManager_AddNavigationContent(self):
        """ NavigationManagers should be able to add NavigationFolders
            and NavigationItems to NavitationRootFolder """
        self.login('navigationmanager')
        self._createNavigation()

    def testNavigationManager_EditNavigationContent(self):
        """ NavigationManagers should be able to add NavigationFolders
            and NavigationItems to NavitationRootFolder.
            
            A different NavigationManager should be able to add and 
            edit content too.
            
            NavigationItems inherit most of their perms from NavigationFolders,
            except ModifyContent permission.
        """

        # NavigationManager 1 will create a hierarchy
        self._createNavigation()
        
        # NavigationManager 2 should be able to add new folders, edit folders, and edit items
        self.login('navigationmanager2')

        # add some new content
        self.rootnavfolder.invokeFactory('NavigationFolder', id='navigation_folder_level1-new')
        self.nav_level_1_folder_A.invokeFactory('NavigationFolder', id='navigation_folder_level2-new')
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='navigation_item_2')
        
        # edit existing content - a folder and an item
        self.nav_level_1_folder_A.setTitle("""New Navigation Folder Title""")
        self.nav_level_1_folder_A.setDescription("""New Description""")
        self.assertEqual(self.nav_level_1_folder_A.Title(),"""New Navigation Folder Title""")
        self.assertEqual(self.nav_level_1_folder_A.Description(),"""New Description""")

        self.nav_item_1_A.setTitle("""New Navigation Item Title""")
        self.nav_item_1_A.setDescription("""New Description""")
        self.nav_item_1_A.setText("""<p>New Text</p>""")
        self.assertEqual(self.nav_item_1_A.Title(),"""New Navigation Item Title""")
        self.assertEqual(self.nav_item_1_A.Description(),"""New Description""")
        self.assertEqual(self.nav_item_1_A.getText(),"""<p>New Text</p>""")

    def testNavigationView(self):
        """ Create navigation hierarchy and test that our python data structure
        reflects that hierarchy.
        """
        request = TestRequest()
        request.RESPONSE = request.response
        self._createNavigation()
        
        # 'publish' some NavigationItems so they appear in the view
        self.workflow_tool.doActionFor(self.nav_item_1_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_2_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_3_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_4_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_5_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_6_A, 'publish')
        # test that 'pending' NavigationItems don't appear
        # self.nav_item_1_B is pending by default
        pass
        # test that 'visible' NavigationItems don't appear
        self.workflow_tool.doActionFor(self.nav_item_2_B, 'reject')
        # test that 'pendingremoval' NavigationItems DO appear
        self.workflow_tool.doActionFor(self.nav_item_3_B, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_3_B, 'requestremoval')
        
        self.logout()

        view = getMultiAdapter((self.portal, request), name='navigation_browserview')
        nav = view.getNavigation()
        
        self.failUnless(len(nav) == 2, '_createNavigation should have ' + \
            '2 level-1 folders.  Instead it has %s' % len(nav))

        #
        # TEST THAT HIERARCHY WAS BUILT CORRECTLY
        #
        
        # test 'A' hierarchy (just checking 'id' fields)
        hierarchy = [ nav[0] ]

        found_externalurl_item = ""
        ones = []
        twos = []
        threes = []
        for one in hierarchy:
            ones.append(one['id'])            
            navLevel2 = one['navLevel2']
            for two in navLevel2:
                twos.append(two['id'])
                # only append navLevel3 if two is a NavigationFolder
                if two['portal_type'] == 'NavigationFolder':
                    navLevel3 = two['navLevel3']
                    for three in navLevel3:
                        threes.append(three['id'])
                        # special check to grab url for 'nav_item_4_A'
                        if three['id'] == 'nav_item_4_A':
                            found_externalurl_item = three['url']
                
        self.failUnless('nav_level_1_folder_A' in ones, 'Missing folder in Level1 in hierarchy A')
        self.failUnless('nav_level_2_folder_A' in twos, 'Missing folder in Level2 in hierarchy A')
        self.failUnless('nav_item_1_A' in threes, 'Missing item in Level3 in hierarchy A')
        self.failUnless('nav_item_2_A' in threes, 'Missing item in Level3 in hierarchy A')
        self.failUnless('nav_item_3_A' in threes, 'Missing item in Level3 in hierarchy A')
        self.failUnless('nav_item_4_A' in threes, 'Missing item in Level3 in hierarchy A')
        self.failUnless('nav_item_5_A' in twos, 'Missing item in Level2 in hierarchy A')
        self.failUnless('nav_item_6_A' in twos, 'Missing item in Level2 in hierarchy A')
        
        # test 'B' hierarchy (just checking 'id' fields)
        hierarchy = [ nav[1] ]

        ones = []
        twos = []
        threes = []
        for one in hierarchy:
            ones.append(one['id'])
            navLevel2 = one['navLevel2']
            for two in navLevel2:
                twos.append(two['id'])
                # only append navLevel3 if two is a NavigationFolder
                if two['portal_type'] == 'NavigationFolder':
                    navLevel3 = two['navLevel3']
                    for three in navLevel3:
                        threes.append(three['id'])
                
        self.failUnless('nav_level_1_folder_B' in ones, 'Missing folder in Level1 in hierarchy B')
        self.failUnless('nav_level_2_folder_B' in twos, 'Missing folder in Level2 in hierarchy B')
        self.failUnless('nav_item_1_B' not in threes, 'Missing item in Level3 in hierarchy B')
        self.failUnless('nav_item_2_B' not in threes, 'Missing item in Level3 in hierarchy B')
        self.failUnless('nav_item_3_B' in threes, 'Missing item in Level3 in hierarchy B')

        # TEST EXTERNAL URL (as setup in _createNavigation())
        self.failUnless(found_externalurl_item == 'http://www.webcollective.coop', \
            "View didn't return proper external url link")
            
        # Does view.getNavigation() return all the proper dictionary items
        # for a NavigationFolder?  Let's grab a folder dictionary.
        hierarchyA = [ nav[0] ]
        folder_dict = hierarchyA[0]
        self.failUnless(len(folder_dict.keys()) == 7, "NavigationFolder dict should have 7 keys.")
        self.failUnless(folder_dict.has_key('id'), "NavigationFolder dict missing 'id' key.")
        self.failUnless(folder_dict.has_key('uid'), "NavigationFolder dict missing 'uid' key.")
        self.failUnless(folder_dict.has_key('title'), "NavigationFolder dict missing 'title' key.")
        self.failUnless(folder_dict.has_key('summary'), "NavigationFolder dict missing 'summary' key.")
        self.failUnless(folder_dict.has_key('portal_type'), "NavigationFolder dict missing 'summary' key.")
        self.failUnless(folder_dict.has_key('placement'), "NavigationFolder dict missing 'placement' key.")
        self.failUnless(folder_dict['placement'] in ("top", "side"), "NavigationFolder dict should have a default value.")
        # TODO? Test navLevel2 and navLevel3 keys

        # Does view.getNavigation() return all the proper dictionary items
        # for a NavigationItem (linking to an internal page URL)
        item_dict = folder_dict['navLevel2'][0]['navLevel3'][0]
        self.failUnless(len(item_dict.keys()) == 6, "NavigationItem dict should have 6 keys.")
        self.failUnless(item_dict.has_key('id'), "NavigationItem dict missing 'id' key.")
        self.failUnless(item_dict.has_key('title'), "NavigationItem dict missing 'title' key.")
        self.failUnless(item_dict.has_key('summary'), "NavigationItem dict missing 'summary' key.")
        self.failUnless(item_dict.has_key('url'), "NavigationItem dict missing 'url' key.")
        self.failUnless(item_dict.has_key('portal_type'), "NavigationItem dict missing 'portal_type' key.")
        self.failUnless(item_dict.has_key('analyticspath'), "NavigationItem dict missing 'analyticspath' key.")
        self.failUnless(item_dict['id'] == 'nav_item_1_A', 'Incorrect id')
        self.failUnless(item_dict['title'] == 'Nav Item 1 A', 'Incorrect title')
        self.failUnless(item_dict['summary'] == '', 'Incorrect summary')
        self.failUnless(item_dict['url'] is None, 'Incorrect url')
        self.failUnless(item_dict['portal_type'] == 'NavigationItem', 'Incorrect portal_type')
        self.failUnless(item_dict['analyticspath'] == '/plone/navigation_root_folder/nav_level_1_folder_A/nav_level_2_folder_A/nav_item_1_A', 'Incorrect analyticspath')
        
        # Test a NavigationItem with an External URL ("nav_item_4_A")
        item_dict = folder_dict['navLevel2'][0]['navLevel3'][3]
        self.failUnless(len(item_dict.keys()) == 6, "NavigationItem dict should have 6 keys.")
        self.failUnless(item_dict.has_key('id'), "NavigationItem dict missing 'id' key.")
        self.failUnless(item_dict.has_key('title'), "NavigationItem dict missing 'title' key.")
        self.failUnless(item_dict.has_key('summary'), "NavigationItem dict missing 'summary' key.")
        self.failUnless(item_dict.has_key('url'), "NavigationItem dict missing 'url' key.")
        self.failUnless(item_dict.has_key('portal_type'), "NavigationItem dict missing 'portal_type' key.")
        self.failUnless(item_dict.has_key('analyticspath'), "NavigationItem dict missing 'analyticspath' key.")
        self.failUnless(item_dict['id'] == 'nav_item_4_A', 'Incorrect id')
        self.failUnless(item_dict['title'] == 'Nav Item 4 A', 'Incorrect title')
        self.failUnless(item_dict['summary'] == '', 'Incorrect summary')
        self.failUnless(item_dict['url'] == 'http://www.webcollective.coop', 'Incorrect url')
        self.failUnless(item_dict['portal_type'] == 'NavigationItem', 'Incorrect portal_type')
        self.failUnless(item_dict['analyticspath'] == '/plone/navigation_root_folder/nav_level_1_folder_A/nav_level_2_folder_A/nav_item_4_A', 'Incorrect analyticspath')
        

    def testNavigationEdit_AddLevelTWONavigationItem(self):
        """ Test creating new NavigationItem for a Document via
        NavigationEdit view. Add to a top-level folder.
        
        NOTE:
            form.navigation_location_top_level must be the title of a valid folder
            form.navigation_item_title is required
        """

        self._createNavigation()
        
        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level': "none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')
        
        myform.update()

        # did the update work?
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)
        # does new NavigationItem exist?
        self.failUnless('my-title' in self.nav_level_1_folder_A.objectIds(), "A new 'my-title' NavigationItem was not created")
        # Does it have the right data?
        navigationitem = self.nav_level_1_folder_A.restrictedTraverse('my-title')
        self.failUnless(navigationitem.id == 'my-title', "Incorrect ID")
        self.failUnless(navigationitem.title == u'My Title', "Incorrect Title")
        self.failUnless(navigationitem.summary == u'My Summary', "Incorrect Summary")
        # Does it refer back to our piece_of_content?
        brefs = piece_of_content.getBRefs(relationship='GlobalMenuTo')
        self.failUnless(len(brefs)==1, 'Not pointing to any content')
        self.failUnless(brefs[0].UID() == navigationitem.UID(), 'Not pointing to correct object')

    def testNavigationEdit_AddLevelTHREENavigationItem(self):
        """ Test creating new NavigationItem for a Document via
        NavigationEdit view. Add to a 2nd-level folder.
        
        NOTE:
            form.navigation_location_top_level must be the title of a valid folder
            form.navigation_item_title is required
        """
        self._createNavigation()
        
        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        #       And form.navigation_location_top_level_secondary_level is a folder UID
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level':self.nav_level_2_folder_A.UID(),
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')
        
        myform.update()

        # did the update work?
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)

        # does the template render ok?
        # myform.render() generates a "Unauthorized: Not authorized to access binding: context" when the
        #       navigation_edit_form has the proper cmf.ModifyPortalContent, but works fine in zope.Public.
        #       Functionality works fine in Plone, but not in the tests so commenting out for now. 
        #self.failUnless(myform.render().find('<body') != -1, 'Template does not have <body> tag.  Did render work ok?')

        # does new NavigationItem exist?
        self.failUnless('my-title' in self.nav_level_2_folder_A.objectIds(), "A new 'my-title' NavigationItem was not created")
        # Does it have the right data?
        navigationitem = self.nav_level_2_folder_A.restrictedTraverse('my-title')
        self.failUnless(navigationitem.id == 'my-title', "Incorrect ID")
        self.failUnless(navigationitem.title == u'My Title', "Incorrect Title")
        self.failUnless(navigationitem.summary == u'My Summary', "Incorrect Summary")
        # Does it refer back to our piece_of_content?
        brefs = piece_of_content.getBRefs(relationship='GlobalMenuTo')
        self.failUnless(len(brefs)==1, 'Not pointing to any content')
        self.failUnless(brefs[0].UID() == navigationitem.UID(), 'Not pointing to correct object')
        
        # test api methods
        self.failUnless(navigationitem.navURL()==piece_of_content.absolute_url(), 'Incorrect URL for content referenced from navigationitem')

        myform.getNavigationItems()

    def testHandleDuplicateNavigationItemTitles(self):
        """ Do we avoid this error:
                Update FAILED due to BadRequest 'The id "abc" is invalid - it is already in use.'
        """
        self._createNavigation()

        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response

        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')

        myform.update()
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)
        first_one = self.nav_level_1_folder_A.restrictedTraverse('my-title')

        # try adding the same item twice
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})        
        myform.update()
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)
        second_one = self.nav_level_1_folder_A.restrictedTraverse('my-title-1')

        # go crazy, try adding the same item three times
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})        
        myform.update()
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)
        third_one = self.nav_level_1_folder_A.restrictedTraverse('my-title-2')
    
    def testGracefullyHandleMissingLocations(self):
        """ What happens it something went wrong with form, and two
        required location fields have no values? """
        self._createNavigation()

        request = TestRequest(form={ 'form.navigation_location_top_level':'',
                                     'form.navigation_location_top_level_secondary_level':'',
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response

        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')

        myform.update()
        self.failUnless(myform.status != 'Updated successfully.', 'Form should have failed. form.status is %s' % myform.status)

        request = TestRequest(form={ 'form.navigation_location_top_level':'none',
                                     'form.navigation_location_top_level_secondary_level':'none',
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})
        myform.update()
        self.failUnless(myform.status != 'Updated successfully.', 'Form should have failed. form.status is %s' % myform.status)
        
    #def testHandleDuplicateNavigationFolderTitles(self):
    #    """ formlib's Choice field doesn't work properly when if NavigationFolders
    #    in the same folder have the same title (even with different IDs)"""
    #    raise AssertionError
        
    def testNavigationEditFormNotAvailableToAnonymous(self):
        """ Test that navigation_edit_form, its subNavigationFolders(),
        and navigation_remove view are not available to Anonymous users"""
        
        self.logout()
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, "@@navigation_edit_form")
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, "@@navigation_edit_form/getSubNavigationFoldersXML")
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, "@@navigation_remove")

    def testGlobalNavigationViewAvailableToAnonymous(self):
        """ Test that globalnavigation_view is available to Anonymous users """
        self.logout()
        self.portal.restrictedTraverse("globalnavigation_view")
                
    def testNavigationEditExternalURL(self):
        """ Test creating new NavigationItem for a Document via
        NavigationEdit view.  This test focuses on if new 'externalurl'
        field works properly.
        
        See testNavigationEdit() for internal content.
        """
        self._createNavigation()
        
        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.navigation_item_external_url':'http://www.webcollective.coop/',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')
        
        myform.update()

        # did the update work?
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)
        
        # does the template render ok?
        # myform.render() generates a "Unauthorized: Not authorized to access binding: context" when the
        #       navigation_edit_form has the proper cmf.ModifyPortalContent, but works fine in zope.Public.
        #       Functionality works fine in Plone, but not in the tests so commenting out for now.
        #self.failUnless(myform.render().find('<body') != -1, 'Template does not have <body> tag.  Did render work ok?')

        # does new NavigationItem exist?
        self.failUnless('my-title' in self.nav_level_1_folder_A.objectIds(), "A new 'my-title' NavigationItem was not created")
        # Does it have the right data?
        navigationitem = self.nav_level_1_folder_A.restrictedTraverse('my-title')
        self.failUnless(navigationitem.id == 'my-title', "Incorrect ID")
        self.failUnless(navigationitem.title == u'My Title', "Incorrect Title")
        self.failUnless(navigationitem.summary == u'My Summary', "Incorrect Summary")
        self.failUnless(navigationitem.externalurl == u'http://www.webcollective.coop/', "Incorrect URL")

        # Make sure it does NOT refer back to our internal piece_of_content
        brefs = piece_of_content.getBRefs(relationship='GlobalMenuTo')
        self.failUnless(len(brefs)==0, 'Should not be pointing to any content')

        self.failUnless(self.workflow_tool.getInfoFor(navigationitem, 'review_state') == 'pending',
            'Content should have been in pending state')

    def test_getSubNavigationFoldersXML(self):
        self._createNavigation()
        request = TestRequest()
        request.RESPONSE = request.response

        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')

        empty_xmlresult = myform.getSubNavigationFoldersXML(topLevelFolderTitle="Non Existing Made-up Folder Title")
        self.failUnless(empty_xmlresult.find('<?xml version="1.0" encoding="UTF-8"?><results><result id="none" title="&lt;no selection&gt;"></result></results>') != -1, 'Should have return empty <results>')

        xmlresult = myform.getSubNavigationFoldersXML(topLevelFolderTitle="Nav Level 1 Folder A")
        self.failUnless(xmlresult == """<?xml version="1.0" encoding="UTF-8"?><results><result id="none" title="&lt;no selection&gt;"></result><result id="%s" title="Nav Level 2 Folder A"></result></results>""" % (self.nav_level_2_folder_A.UID()), 'Should have returned different <results>')
        

    def testNavigationEditBADExternalURL(self):
        """ Test creating new NavigationItem for a Document via
        NavigationEdit view.  This test focuses on if from properly catches
        problem if URL is not valid.
        """

        self._createNavigation()
        
        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A / Nav Level 2 Folder A',
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'My Title',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.navigation_item_external_url':'BAD URL',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')
        
        myform.update()

        # a bad URL should prevent the NavigationItem from being created
        self.assertRaises(AttributeError, self.nav_level_2_folder_A.restrictedTraverse, 'my-title')
        
    def testNavigationFolderVocabulary(self):
        """ Test our vocabulary utility """
        
        request = TestRequest()
        request.RESPONSE = request.response
        self._createNavigation()
        
        utility = getUtility(INavigationFolderVocabulary)

        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        vocab_iter = SimpleVocabulary.fromItems(utility.navigation_folders(piece_of_content, request))

        # convert iterator into a list of tuples
        vocab = []
        for term in vocab_iter:
            vocab.append( (term.token, term.value, ) )
        
        self.failUnless(len(vocab) == 3, 'Not all NavigationFolders were returned')
        self.failUnless(vocab[0][0] == 'Please choose a button...')
        self.failUnless(vocab[1][0] == 'Nav Level 1 Folder A', 'First folder not correct')
        self.failUnless(vocab[2][0] == 'Nav Level 1 Folder B', 'First folder not correct')

    def testNavigationItem_NavigationManagerCreateAndPublish(self):
        """ NavigationItems start as 'pending', then should be able to be
        published by NavigationManagers.
        """
        self._createNavigation()
        self.login('navigationmanager')
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='navitem',
                title="Nav Item")
        navitem = self.nav_level_2_folder_A.restrictedTraverse('navitem')

        # new NavigationItem should start in pending mode
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'pending',
            "NavigationItem should start in 'pending' state")
        # can we Publish?
        self.workflow_tool.doActionFor(navitem, 'publish')
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'published',
            "NavigationItem should have been published")

    def testNavigationItem_NavigationManagerRemovalProcess(self):
        """ NavigationItems go to 'request removal' prior to removal.
        """
        self._createNavigation()
        self.login('navigationmanager')
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='navitem',
                title="Nav Item")
        navitem = self.nav_level_2_folder_A.restrictedTraverse('navitem')

        # start with published item
        self.workflow_tool.doActionFor(navitem, 'publish')
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'published',
            "NavigationItem should have been published")
            
        # run the 'requestremoval' action, which should move navitem into 'pending removal' state
        self.workflow_tool.doActionFor(navitem, 'requestremoval')
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'pendingremoval',
            "NavigationItem should have been 'pending removal'")

        # now an Admin can either delete it, 'reject' or 'publish'
        #    let's try to publish, move back to requestremoval, then try reject
        self.workflow_tool.doActionFor(navitem, 'publish')
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'published',
            "NavigationItem should have been 'pending removal'")

        self.workflow_tool.doActionFor(navitem, 'requestremoval')
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'pendingremoval',
            "NavigationItem should have been 'pending removal'")

        self.workflow_tool.doActionFor(navitem, 'reject')
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'visible',
            "NavigationItem should have been 'pending removal'")
        
    def testNavigationItemDeleteRemovesReferences(self):
        """ When a NavigationManager deletes a NavigationItem, does the reference catalog
        get cleaned up appropriately?
        """
        # TODO Write this test
        #n.deleteReference(self.context, relationship='GlobalMenuTo')
    
    def testNavigationRemoveView(self):
        """ Ensure removal works, plus make sure it handles passing of crazy parameters.
            Also check that Anonymous can not call this view.
        """
        # create test nav hierarchy
        request = TestRequest()
        request.RESPONSE = request.response
        
        self._createNavigation()
        self.nav_level_2_folder_A.invokeFactory('NavigationItem', id='navitem',
                title="Nav Item")
        navitem = self.nav_level_2_folder_A.restrictedTraverse('navitem')
        self.workflow_tool.doActionFor(navitem, 'publish')

        self.logout()
        view = getMultiAdapter((navitem, request), name='navigation_remove')

        # grab NavigationRemoveView
        self.login('navigationmanager')
        view = getMultiAdapter((navitem, request), name='navigation_remove')

        # call view for this UID
        request.form['uid'] = navitem.UID()
        view()

        # was the object's workflow marked as 'pendingremoval'?
        self.assertEqual(self.workflow_tool.getInfoFor(navitem, 'review_state'), 'pendingremoval',
            "NavigationItem should have been 'pending removal'")

    def testNavigationPortletView(self):
        # create test nav hierarchy
        request = TestRequest()
        request.RESPONSE = request.response
        
        self._createNavigation()
        
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')
        
        # grab NavigationRemoveView
        self.logout()
        view = getMultiAdapter((piece_of_content, request), name='navigation_portlet')
        self.failUnless(len(view.getNavItemsForReview())==9, "There should have been 7 pending NavigationItems")
        
        # grab the first one and make sure we're getting the dictionary we expect
        navitem = view.getNavItemsForReview()[0]
        self.failUnless(hasattr(navitem, 'portal_type'), "Portlet: Dictionary missing key 'portal_type'")
        self.failUnless(hasattr(navitem,'getURL'), "Portlet: Dictionary missing method 'getURL'")
        self.failUnless(hasattr(navitem,'Description'), "Portlet: Dictionary missing key 'Description'")
        self.failUnless(hasattr(navitem,'pretty_title_or_id'), "Portlet: Dictionary missing key 'pretty_title_or_id'")
        self.failUnless(hasattr(navitem, 'ModificationDate'), "Portlet: Dictionary missing key 'ModificationDate'")
        self.failUnless(hasattr(navitem, 'review_state'), "Portlet: Dictionary missing key 'review_state'")

        self.login('navigationmanager')

        # publish 3
        self.workflow_tool.doActionFor(self.nav_item_1_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_2_A, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_3_A, 'publish')
        # the 4th is nav_item_4_A (for testing External URL field)
        self.failUnless(len(view.getNavItemsForReview())==6, "There should now only be 6 pending NavigationItems")
        
        # make 3 'pendingremoval' (so should show up just like the 'pending' ones)
        self.workflow_tool.doActionFor(self.nav_item_1_B, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_2_B, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_3_B, 'publish')
        self.workflow_tool.doActionFor(self.nav_item_1_B, 'requestremoval')
        self.workflow_tool.doActionFor(self.nav_item_2_B, 'requestremoval')
        self.workflow_tool.doActionFor(self.nav_item_3_B, 'requestremoval')
        # the 4th is nav_item_4_A (for testing External URL field)
        self.failUnless(len(view.getNavItemsForReview())==6, "There should now only be 6 pending NavigationItems")        
        
    def testCatalogNavURL(self):
        """ Tests new navURL catalog index """
        
        self._createNavigation()

        # search for nav_item_4_A (external url) in catalog and make sure navURL is correct
        navitems = self.portal_catalog.searchResults(
                              id = 'nav_item_4_A',
                              portal_type = ['NavigationItem',],
                              )
        self.failUnless(navitems[0].navURL == self.nav_item_4_A.getExternalurl(),'navURL in catalog is not correct')

        #
        # search for nav_item_3_A (non-external url) in catalog and make sure navURL is correct
        #
        request = TestRequest(form={ 'form.navigation_location_top_level':'Nav Level 1 Folder A',
                                     'form.navigation_location_top_level_secondary_level':"none",
                                     'form.navigation_item_title':'Test navURL',
                                     'form.navigation_item_summary':'My Summary',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        # create content that has the IATContentType interface
        self.login()
        self.folder.invokeFactory('Document', id='piece_of_content', title="Piece of Content")
        piece_of_content = self.folder.restrictedTraverse('piece_of_content')

        myform = getMultiAdapter((piece_of_content, request), name='navigation_edit_form')
        
        myform.update()
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of %s' % myform.status)

        navigationitem = self.nav_level_2_folder_A.restrictedTraverse('test-navurl')
        navigationitem.reindexObject()

        # Does it refer back to our piece_of_content?
        brefs = piece_of_content.getBRefs(relationship='GlobalMenuTo')
        self.failUnless(len(brefs)==1, 'Not pointing to any content')
        self.failUnless(brefs[0].UID() == navigationitem.UID(), 'Not pointing to correct object')
        
        
        navitems = self.portal_catalog.searchResults(
                              id = 'test-navurl',
                              portal_type = ['NavigationItem',],
                              )
        self.failUnless(navitems[0].navURL == piece_of_content.absolute_url(),'navURL in catalog is not correct')

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestRolesAndWorkflow))
    return suite
