# Integration tests specific to metanav
#

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.metanav.tests import base
from Products.CMFCore.utils import getToolByName
from Products.metanav.config import ALLTYPES, ALLSKINS

class TestProductUnInstallation(base.MetaNavTestCase):
    """ Ensure that our product plays nicely and uninstalls correctly.
    """
    def afterSetUp(self):
        self.types      = self.portal.portal_types
        self.properties = self.portal.portal_properties
        self.factory    = self.portal.portal_factory
        self.skins      = self.portal.portal_skins
        self.qi         = self.portal.portal_quickinstaller
        self.workflow   = self.portal.portal_workflow
        
        self.metanavTypes  = ALLTYPES 
        self.metanavTypeSkins = ALLSKINS
        
        # uninstall our product
        if self.qi.isProductInstalled('metanav'):
            self.qi.uninstallProducts(products=['metanav',])
    
    def testQIDeemsProductUninstalled(self):
        """Make sure the product is uninstalled in the eyes of the portal_quickinstaller"""
        self.failIf(self.qi.isProductInstalled('metanav'))
    
    def testAdapterTypeNotRegisteredOnUninstall(self):
        for t in self.metanavTypes:
            self.failIf(t in self.types.objectIds(),
                "Type %s is still registered with the types tool after uninstallation." % t)
    
    def testFactoryTypesRemovedOnUninstall(self):
        for t in self.metanavTypes:
            self.failIf(t in self.factory.getFactoryTypes(),
                "Type %s is still a factory type after uninstallation." % t)
    
    def testTypesNotSearchedRemovedOnUninstall(self):
        # TODO - revisit me
        pass
    
    def testTypesNotListedRemovedOnUninstall(self):
        metaTypesNotToList  = self.properties.navtree_properties.getProperty('metaTypesNotToList')
        for t in self.metanavTypes:
            self.failIf(t in metaTypesNotToList,
                "Type %s is still in the list of metaTypesNotToList after uninstallation" % t)
    
    def testSkinsRemovedOnUninstall(self):
        """Our all important skin layer(s) should be registered with the site."""
        
        for prodSkin in self.metanavTypeSkins:
            self.failIf(prodSkin in self.skins.objectIds(),
                "The skin %s is still registered with the skins tool after uninstallation" % prodSkin)
    
    def testSkinLayersRemovedOnUninstall(self):
        """We need our product's skin directories to show up below custom as one of the called
           upon layers of our skin's properties
        """
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in self.metanavTypeSkins:
                self.failIf(specific_layer in layers, "The %s layer \
                    still appears in the layers of Plone's %s skin after uninstall" % (specific_layer,selection))
    
    #def testWorkflowToolNoLongerMaintainsWorkflowChainForRemovedType(self):
    #    for t in self.metanavTypes:
    #        self.failIf(self.workflow._chains_by_type.has_key(t), "The %s type \
    #            still appears to have a workflow chain with the workflow tool" % t)

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductUnInstallation))
    return suite
